#ifndef _PCI_32_64_H
#define _PCI_32_64_H

#include "PCI.h"
#include "PCI_Bus.h"

#include <windows.h>
#include "windrvr.h"

namespace EARTH {
namespace PCI {
	namespace _32 { status NewBus(Bus **); }
	namespace _64 { status NewBus(Bus **); }

	class Device;
	namespace _32 { status NewDevice(HANDLE, const Target *, Device **); }
	namespace _64 { status NewDevice(HANDLE, const Target *, Device **); }

	class Memory;
	namespace _32 { status NewMemory(HANDLE, DWORD, Memory **); }
	namespace _64 { status NewMemory(HANDLE, DWORD, Memory **); }

	class _Top : public Top {
	public:
		_Top(uint32 kernelBit) : mKernelBit(kernelBit)
		{
			mBus = NULL;
		}

		virtual ~_Top()
		{
			delete_(mBus);
		}

		virtual status Init()
		{
			_ASSERT(mBus == NULL);

			status	status;

			if (mKernelBit == 32) {
				status = _32::NewBus(&mBus);
			} else {
				status = _64::NewBus(&mBus);
			}

			return status;
		}

		virtual class Bus *Bus() const
		{
			return mBus;
		}

		virtual status NewDeviceMemory(const Target *target, Device **device, Memory **memory)
		{
			_ASSERT(target);

			_ASSERT(mBus);
			HANDLE	handle = mBus->Handle();
			if (handle == INVALID_HANDLE_VALUE) return PT::STATUS_INTERNAL_ERROR;

			status	status;

			if (mKernelBit == 32) {
				status = _32::NewDevice(handle, target, device);
				if (status) return status;

				status = _32::NewMemory(handle, (*device)->ID(), memory);
			} else {
				status = _64::NewDevice(handle, target, device);
				if (status) return status;

				status = _64::NewMemory(handle, (*device)->ID(), memory);
			}

			return status;
		}

	protected:
		const uint32	mKernelBit;
		class Bus		*mBus;
	};
}
}

#endif
